
import java.util.ArrayList;
import java.util.Collection;

/**
 * This constraint makes the end-point of an arc stay on the circle as defined by the start-point.
*/

public class SCArcConstraint extends SConstraint {
	/**
	 * A constant factor by which the error is multiplied, in order to make this constraint higher-priority.
	*/
	private static int errorMult = 4;

	/**
	 * The constructor.
	*/
	public SCArcConstraint(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
		//required are: 1 arc
		super(owner, refs, 0, 0, 1, 0, additional_points);
	}
	
	/**
	 * @returns an integer array representing the number of points, lines and arcs required
	 */
	public static int[] getRequired()
	{
		int[] reqShapes = {0,0,1,0};
		return (reqShapes);
	}
	
	/**
	 * The function used to calculate the error values.
	*/
	public int calcError(int values[]) {
		/*
		The values fields should be filled as follows:
		0/1: the centre
		2/3: the start-point
		4/5: the end-point
		*/
		
		int dx1 = values[0] - values[2]; //x distance in-between the centre and start-point
		int dx2 = values[0] - values[4]; //x distance in-between the centre and end-point
		int dy1 = values[1] - values[3]; //y distance in-between the centre and start-point
		int dy2 = values[1] - values[5]; //y distance in-between the centre and end-point
		
		return java.lang.Math.abs((int) (pythagoras(dx1, dy1) - pythagoras(dx2, dy2))) * errorMult;
	}
	
	/**
	 * The formula of pythagoras in one handy shiny function.
	*/
	public static double pythagoras(int dx, int dy) {
		return java.lang.Math.sqrt((double) (dx*dx + dy*dy));
	}
		

	/**
	 * Returns the collection of variables this constraint applies to.
	 * This constraint binds the end-point only.
	*/
	public ArrayList myVariables() {
		ArrayList ret = new ArrayList();
		
		ret.add(allVariables[4]);
		ret.add(allVariables[5]);
		
		return ret;
	}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "Ø";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "Ø";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Internal use only: makes sure end-point stays on circle arc.";
	}

}
