import java.util.Collection;
import java.util.ArrayList;

/**
 * This constraint specifies that the distance in between two points be x times the distance in between two others.
 * Note: It always assumes the second set of points to be x times as far apart. For a divider, reverse input order!
*/

public class SCDistanceFactor extends SConstraint {
	/**
	 * The constructor.
	*/
	public SCDistanceFactor(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
		//required are: 4 points, one variable
		super(owner, refs, 4, 0, 0, 1, additional_points);
	}
	
	/**
	 * @returns an integer array representing the number of points, lines and arcs required
	 */
	public static int[] getRequired()
	{
		int[] reqShapes = {4,0,0,1};
		return (reqShapes);
	}
	
	/**
	 * The function used to calculate the error values.
	*/
	public int calcError(int values[]) {
		/*
		The values fields should be filled as follows:
		0-7: the x/y coordinates of the 4 points, in the order they came in.
		8: the variable that determines the scale factor
		*/
		
		int dx1 = values[0] - values[2]; //x distance in-between the first two points
		int dx2 = values[4] - values[6]; //x distance in-between the second two points
		int dy1 = values[1] - values[3]; //y distance in-between the first two points
		int dy2 = values[5] - values[7]; //y distance in-between the first second points
		
		return java.lang.Math.abs((int) (pythagoras(dx1, dy1) - pythagoras(dx2, dy2) * values[8]));
	}
	
	/**
	 * The formula of pythagoras in one handy shiny function.
	*/
	public static double pythagoras(int dx, int dy) {
		return java.lang.Math.sqrt((double) (dx*dx + dy*dy));
	}
	
	/**
	* This is over-ridden from the main class because the distance variable is not bound by this constraint.
	* It is for reference only.
	* @returns the collection of variables this constraint applies to
	*/
	public ArrayList myVariables() {
		ArrayList ret = new ArrayList();

		for (int i = 0; i < 8; i++) {
			ret.add(allVariables[i]);
		}
	
		return ret;
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "M";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "M";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Select four points and a number label. This constraint makes the distance in-between " +
				"the second pair of points equal to the distance in-between the first pair, " +
               "multiplied by the specified number.";
	}
	
}
