import java.util.Collection;
import java.util.ArrayList;
import java.lang.Math;

/**
 * Demands that the distance in-between 2 points remain constant.
 *
 * @author David Stark
 * @version 1.0
*/
public class SCFixedDistance extends SConstraint{

/**
 * The constructor.
 *
 * @param refs the shapes this constraint should refer to
*/
public SCFixedDistance(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
	super(owner, refs, 2, 0, 0, 0, additional_points);
	
	//after calling the constructor, calculate the distance in between the two chosen points
	//and add it as an extra variable
	
	SVariable temp[] = allVariables;
	
	int dx = temp[0].value() - temp[2].value();
	int dy = temp[1].value() - temp[3].value();
	int d = (int) Math.abs( Math.sqrt((double) ((dx*dx + dy*dy))));
	
	SVariable distance = new SVariable(d);
	
	SVariable newVars[] = new SVariable[5];
	
	newVars[0] = temp[0];
	newVars[1] = temp[1];
	newVars[2] = temp[2];
	newVars[3] = temp[3];
	newVars[4] = distance;
	
	allVariables = newVars;
	theVariables = new SVariable[1];
	theVariables[0] = distance;
}

/**
 * @returns an integer array representing the number of points, lines and arcs required
 */
public static int[] getRequired()
{
	int[] reqShapes = {2,0,0,0};
	return (reqShapes);
}

/**
 * The function to calculate the error of this constraint for the given variables.
*/
public int calcError(int values[]) {
	int dx = values[0] - values[2];
	int dy = values[1] - values[3];
	return (int) Math.abs( Math.sqrt((double) ((dx*dx + dy*dy))) - values[4]);
}

/**
 * This is over-ridden from the main class because the distance variable is not bound by this constraint.
 * It is for reference only.
 * @returns the collection of variables this constraint applies to
*/
public ArrayList myVariables() {
	ArrayList ret = new ArrayList();

	for (int i = 0; i < 4; i++) {
		ret.add(allVariables[i]);
	}
	
	return ret;
}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "F";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "F";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Click on two points for which you want the separation to remain constant.";
	}

}
