import java.util.Collection;
import java.util.ArrayList;

/**
 * This constraint specifies that the value of the passed variable is 256 times the quotient of the distances
 * in between two points each. This is to define the relationship in between attachers and the scale variable
 * of instances.
*/

public class SCInstanceScaleFactorRelation extends SConstraint{
	/**
	 * The constructor.
	*/
	public SCInstanceScaleFactorRelation(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
		//required are: 4 points, one variable
		//coords in instance, coords in original, scalefactor
		super(owner, refs, 4, 0, 0, 1, additional_points);
	}
	
	/**
	 * @returns an integer array representing the number of points, lines and arcs required
	 */
	public static int[] getRequired()
	{
		int[] reqShapes = {4,0,0,1};
		return (reqShapes);
	}
	
	/**
	 * The function used to calculate the error values.
	*/
	public int calcError(int values[]) {
		/*
		The values fields should be filled as follows:
		0-7: the x/y coordinates of the 4 points, in the order they came in.
		8: the variable that determines the scale factor
		*/
		
		int dx1 = values[0] - values[2]; //x distance in-between the first two points
		int dx2 = values[4] - values[6]; //x distance in-between the second two points
		int dy1 = values[1] - values[3]; //y distance in-between the first two points
		int dy2 = values[5] - values[7]; //y distance in-between the first second points
		
		return java.lang.Math.abs((int) (values[8] - (256 * pythagoras(dx1, dy1) / pythagoras(dx2, dy2))));
	}
	
	/**
	 * The formula of pythagoras in one handy shiny function.
	*/
	public static double pythagoras(int dx, int dy) {
		return java.lang.Math.sqrt((double) (dx*dx + dy*dy));
	}
	
/**
 * @returns the collection of variables this constraint applies to
*/
public ArrayList myVariables() {
	ArrayList ret = new ArrayList();

	for (int i = 0; i < 4; i++) {
		ret.add(allVariables[i]);
	}
	
	ret.add(allVariables[8]);
	
	return ret;
}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "$";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "$";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
	    // No need for a description of this constraint because the user doesn't actually apply this
	    // it is for the benefit of the constraint system
		return "Internal Use Only. Keeps scale factor of instances correct.";
	}

}
