/**
 * This is a constraint that measures the parallelity of two lines or instances.
 *
 * @author David Stark
 * @version 3
*/
public class SCParallel extends SConstraint {

/**
 * The constructor.
 *
 * @param refs the shapes this constraint should refer to
*/
public SCParallel(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException{
	super(owner, refs, 0, 2, 0, 0, additional_points);
}

/**
 * @returns an integer array representing the number of points, lines and arcs required
 */
public static int[] getRequired()
{
	int[] reqShapes = {0,2,0,0};
	return (reqShapes);
}

/**
 * The function to calculate the error of this constraint for the given variables.
*/
public int calcError(int values[]) {
	/*
	Error calculation is done by calculating the surface of the parallelogram indicated by the 4 points
	of the line, with the first point of each line considered to be "the same".
	The second point of the second line is shifted so that its relation to the first point of the
	first line is equal to its original relation to the first point of the second line.
	In this way we reduce the problem to collinearity.
	*/
	/*
	int p2x2 = values[6] - values[4] + values[0];
	int p2y2 = values[7] - values[5] + values[1];
	
	return java.lang.Math.abs(values[0]*(values[3] - p2y2) + values[2]*(p2y2-values[1]) + p2x2*(values[1]-values[3]));
	*/
	
	int foo = java.lang.Math.abs( (values[2] - values[0]) * (values[7] - values[5]) - (values[6] - values[4]) * (values[3] - values[1]) ) / 1000;
	
	return foo;
}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "P";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "P";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Select two lines, and the constraint will make them parallel.";
	}
	
}
