/**
 * This constraint makes two lines have the same length. (Or two instances have the same distance in attachers,
 * though that's rather meaningless and arbitrary if the instances are of different things.)
*/

public class SCSameLength extends SConstraint{
	/**
	 * The constructor.
	*/
	public SCSameLength(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
		//required are: 2 lines
		super(owner, refs, 0, 2, 0, 0, additional_points);
	}
	
	/**
	 * @returns an integer array representing the number of points, lines and arcs required
	 */
	public static int[] getRequired()
	{
		int[] reqShapes = {0,2,0,0};
		return (reqShapes);
	}
	
	/**
	 * The function used to calculate the error values.
	*/
	public int calcError(int values[]) {
		/*
		The values fields should be filled as follows:
		0-7: the x/y coordinates of the 4 points of the 2 lines, in the order they came in.
		*/
		
		int dx1 = values[0] - values[2]; //x distance in-between the first two points
		int dx2 = values[4] - values[6]; //x distance in-between the second two points
		int dy1 = values[1] - values[3]; //y distance in-between the first two points
		int dy2 = values[5] - values[7]; //y distance in-between the first second points
		
		return java.lang.Math.abs((int) (pythagoras(dx1, dy1) - pythagoras(dx2, dy2)));
	}
	
	/**
	 * The formula of pythagoras in one handy shiny function.
	*/
	public static double pythagoras(int dx, int dy) {
		return java.lang.Math.sqrt((double) (dx*dx + dy*dy));
	}
		
		
	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "lce";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "e";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Select two lines and the system will make them the same length.";
	}

}
