import java.util.ArrayList;
import java.util.Collection;

/**
 * This constraint makes two points be at the same x-coordinate.
 *
 * @author David Stark
 * @version 3.1
*/
public class SCSameX extends SConstraint {

/**
 * The constructor.
 *
 * @param owner the SFile this constraint belongs to
 * @param refs the shapes this constraint should refer to
 * @param any additional points this constraint should extend a line to when being rendered
*/
public SCSameX(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
	super(owner, refs, 2, 0, 0, 0, additional_points);
}

/**
 * @returns an integer array representing the number of points, lines and arcs required
 */
public static int[] getRequired()
{
	int[] reqShapes = {2,0,0,0};
	return (reqShapes);
}

/**
 * The function to calculate the error of this constraint for the given variables.
 * It does this by calculating the difference in-between the x-coordinates of the two given points.
*/
public int calcError(int values[]) {
	return java.lang.Math.abs(values[0] - values[2]);
}

/**
 * Returns the collection of variables this constraint applies to.
 *
 * @returns the collection of variables this constraint applies to
*/
public ArrayList myVariables() {
	ArrayList ret = new ArrayList();

	ret.add(allVariables[0]);
	ret.add(allVariables[2]);
	
	return ret;
}

/**
* @returns the symbol representing this constraint
*/
public static String getSymbol()
{
	return "X";
}
	
/**
* @returns the symbol representing this constraint
*/
public String getSymbolDyn()
{
	return "X";
}

/**
* @returns a description of the constraint
*/
public static String getDescription()
{
	return "Select two points to make their x-coordinate the same.";
}

}
