import java.util.Collection;
import java.util.ArrayList;
import java.lang.Math;

/**
 * This is a "n distance" constraint that instructs two points to be a variable-set distance from each other.
 *
 * @author David Stark
 * @version 3.0
*/
public class SCSetDistance extends SConstraint{

/**
 * The constructor.
 *
 * @param refs the shapes this constraint should refer to
*/
public SCSetDistance(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
	super(owner, refs, 2, 0, 0, 1, additional_points);
}

/**
 * @returns an integer array representing the number of points, lines and arcs required
 */
public static int[] getRequired()
{
	int[] reqShapes = {2,0,0,1};
	return (reqShapes);
}

/**
 * The function to calculate the error of this constraint for the given variables.
*/
public int calcError(int values[]) {
	int dx = values[0] - values[2];
	int dy = values[1] - values[3];
	return (int) Math.abs( Math.sqrt((double) ((dx*dx + dy*dy))) - values[4]);
}

/**
 * This is over-ridden from the main class because the distance variable is not bound by this constraint.
 * It is for reference only.
 * @returns the collection of variables this constraint applies to
*/
public ArrayList myVariables() {
	ArrayList ret = new ArrayList();

	for (int i = 0; i < 4; i++) {
		ret.add(allVariables[i]);
	}
	
	return ret;
}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "D";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "D";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Select two points and a number label and the constraint will keep the distance" +
		       " in-between the points equal to the number.";
	}	
	

}
