import java.util.Collection;
import java.util.ArrayList;

/**
 * This constraint demands that a 2-point object have a given distance in between the 2 points.
 * Essentially the same as SCSetDistance.
*/

public class SCSetSize extends SConstraint{

/**
 * The constructor.
*/
public SCSetSize(SFile owner, java.util.ArrayList refs, java.util.ArrayList additional_points) throws SConstraintCouldNotBeCreatedException {
	//this one requires a single 2-point object and a variable
	super(owner, refs, 0, 1, 0, 1, additional_points);
}

/**
 * @returns an integer array representing the number of points, lines and arcs required
 */
public static int[] getRequired()
{
	int[] reqShapes = {0,1,0,1};
	return (reqShapes);
}

/**
 * The function to calculate the error of this constraint for the given variables.
*/
public int calcError(int values[]) {
	int dx = values[0] - values[2];
	int dy = values[1] - values[3];
	return (int) Math.abs( Math.sqrt((double) ((dx*dx + dy*dy))) - values[4]);
}

/**
 * This is over-ridden from the main class because the distance variable is not bound by this constraint.
 * It is for reference only.
 * @returns the collection of variables this constraint applies to
*/
public ArrayList myVariables() {
	ArrayList ret = new ArrayList();

	for (int i = 0; i < 4; i++) {
		ret.add(allVariables[i]);
	}
	
	return ret;
}

	/**
 	* @returns the symbol representing this constraint
 	*/
	public static String getSymbol()
	{ return "lcl";
	}
	
	/**
 	* @returns the symbol representing this constraint
 	*/
	public String getSymbolDyn()
	{ return "l";
	}

	/**
 	* @returns a description of the constraint
 	*/
	public static String getDescription()
	{
		return "Select a line and a number label. The constraint will keep the line's " +
			   "length equal to the number.";
	}
}