import java.awt.*;
import java.awt.event.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import javax.swing.*;

/*
 * Created on 27-Feb-2005
 *
*/
/**
 * @author mdob2
 */
public class SCanvas extends JPanel implements MouseListener, MouseMotionListener, KeyListener, DropTargetListener{
	
	private boolean rightClick;
	private boolean dragging;
	private SketchUI parent;

	Toolkit tk = Toolkit.getDefaultToolkit();
	Image zoomInImage = (new ImageIcon("images/cursormag.gif")).getImage();
	Image zoomOutImage = (new ImageIcon("images/cursormagout.gif")).getImage();
	Cursor zoomin = tk.createCustomCursor(zoomInImage, new Point(5,5), "Blah");
	Cursor zoomout = tk.createCustomCursor(zoomOutImage, new Point(5,5), "Blah");

	public SCanvas(SketchUI parent)
	{
		this.parent = parent;
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
		this.addKeyListener(this);
		DropTarget dt = new DropTarget(this, this);
		
		
	}

	/**
	* Zoom in or out depending on the current setting
	*/	
	public void setMagnify(int state)
	{
		if(state==0) this.setCursor(java.awt.Cursor.getPredefinedCursor(java.awt.Cursor.DEFAULT_CURSOR));
		if(state==1) this.setCursor(zoomin);
		if(state==2) this.setCursor(zoomout);
	}
	

	/*
	 * tells file system whenever a mouse click, release occurs
	 * updating cursor position of where it happened 
	 */  
	public void mouseClicked (MouseEvent m)
	{
	}
	public void mousePressed(MouseEvent m) 
	{
		rightClick = false;
		dragging = true;
		if(m.isPopupTrigger()) rightClick = true; 
	}


	public void mouseReleased(MouseEvent m)
	{
		if(m.isPopupTrigger()) rightClick = true; 
		parent.mouseClick(m,rightClick);
		dragging = false;
	}
	
	
//scrolling
	
/*
 * tells file system when mouse is dragged, or if it moves over 
 * canvas.  passes coordinates of event and calls refresh on canvas
 * increases the drawing area if mouse is dragged within 100 pixels
 * of the edge of the page's area, or scrolls if dragged at the 
 * edge of the visible part of the canvas  
 */		
	
	public void mouseDragged(MouseEvent m)
	{
		parent.mouseDrag(m);
	} 

	//constantly update filesystem with position of cursor
	//and call refresh on picture

	public void mouseMoved(MouseEvent m) 
	{
		parent.mouseMove(m);
	}
	
	public void mouseExited (MouseEvent m){}
	public void mouseEntered (MouseEvent m){}
	
	/*
	 * allows the user to select multiple items for
	 * constraints, deletion etc by pressing the SHIFT
	 * key while using the 'Cursor' tool then using the
	 * desired tool
	 */


	public void keyTyped (KeyEvent k){};
	public void keyPressed(KeyEvent k)
	{
		System.out.println("key pressed");
		if(k.getKeyCode() == KeyEvent.VK_SHIFT)
		{
			parent.selectMore(true);
		}
		else if(k.getKeyCode() == KeyEvent.VK_SPACE)
		{
			parent.doConstraints();
		}
		else if(k.getKeyCode() == KeyEvent.VK_DELETE)
		{
			parent.deleteShapes();
		}
		
	}
	public void keyReleased(KeyEvent k)
	{
		if(k.getKeyCode()== KeyEvent.VK_SHIFT)
		{
			parent.selectMore(false);
		}
	}
	
	public void dragEnter (DropTargetDragEvent de)
	{
		System.out.println(de);
		de.acceptDrag(de.getSourceActions());
	}
	
	public void dragExit (DropTargetEvent de){}
	public void dragOver (DropTargetDragEvent de){}
	
	/*
	* dropping code - check if drop has been invoked by the library panel and if so
	* notify SketchUI of the drag operation
	*/ 
	public void drop (DropTargetDropEvent e)
	{
		int acceptableActions = DnDConstants.ACTION_COPY;
		DataFlavor chosen = DataFlavor.stringFlavor;
		
		Object data=null;
		try
		{
			e.acceptDrop(acceptableActions);
			data = e.getTransferable().getTransferData(chosen);
			if (data == null) throw new NullPointerException();
	    }
		catch (Throwable t)
		{
			System.err.println( "Couldn't get transfer data: " + t.getMessage());
			e.dropComplete(false);    		
			return;
	    }
	      
		if (data instanceof String)
		{
			try{ 
				parent.fileDropped(e.getLocation().x,e.getLocation().y,(String) data);
				
			}
			catch(SCyclicInstancesException ce)
			{
				JOptionPane.showMessageDialog(new JFrame(), "Dropping this instance into this file would cause a cycle", "Error", JOptionPane.ERROR_MESSAGE, null);
			}
		}
		else
		{
			e.dropComplete(false);      	  	
			return;
	    }
	    e.dropComplete(true);          	      
	}
	
	
	public void dropActionChanged (DropTargetDragEvent de){};

	public boolean isDragging()
	{
		return dragging;
	}

	
}
