import java.io.Serializable;

/**
 * A convenience class to avoid keeping passing the same two SVariables around the place.
 *
 * @author David Stark, Tom Craig
 */
public class SCoords implements Cloneable, Serializable
{
	/**
	 * The page co-ordinates.
	 */
	private SVariable x;
	private SVariable y;
	
	/**
	 * A general constructor.
	 */
	public SCoords(int x, int y) {
		this.x = new SVariable(x);
		this.y = new SVariable(y);
	}
	
	public SCoords(SIntPair def) {
		this.x = new SVariable(def.x);
		this.y = new SVariable(def.y);		
	}
	
	public SVariable getX() {
		return x;
	}
	
	public SVariable getY() {
		return y;
	}
	
	public void print()
    {
        System.out.print(this.toString());
    }
    
    public String toString()
    {
    	return "(" + Integer.toString(x.value()) + "," + Integer.toString(y.value()) + ")";
    }
	
	/**
	 * @return An SIntPair object representing these co-ordinates.
	 */
	public SIntPair makeIntPair()
	{
	    return new SIntPair(x.value(), y.value());
	}
	
	/**
	 * @param c The other pair of co-ordinates.
	 * @return The shortest distance between these co-ordinates and c.
	 */
	public double distanceBetweenCoords(SCoords c)
	{
	    int dx = x.value() - c.x.value();
	    int dy = y.value() - c.y.value();
	    double squaredDist = dx * dx + dy * dy;
	    return Math.sqrt(squaredDist);
	}
	
	/**
    * @returns object reference for cloned object
    */
    public Object clone()
    {
    	SCoords cln = null;
    	try {
    		cln = (SCoords) super.clone();
    		System.out.println("CLONING SCOORDS");
    	}
    	catch (CloneNotSupportedException E) {
    		// SConstraint has a clone method so should never reach here
    		System.out.println("Unexpected Error Cloning SCoords");
    	}
    	
    	cln.x = (SVariable) this.x.clone();
    	cln.y = (SVariable) this.x.clone();
    	
    	return cln;
    }
    
    /**
     * Compare the integer values of the variables for equality.
     * @return True iff. the x-values are equal and the y-values are equal.
     */
    public boolean equals(Object o)
    {
        if (!(o instanceof SCoords))
            return false;
        
        SCoords c = (SCoords) o;
        return (this.getX().value() == c.getX().value() && this.getY().value() == c.getY().value());
    }
        
}