import java.awt.image.BufferedImage;
import java.awt.Color;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import javax.imageio.ImageIO;

/**
 * @author Tom Craig
 */
public abstract class SDataTest
{
    public static void main(String args[])
    {
        while (SDrawSketch.ZoomIn());
        
        SDataTest testObject = new STCircuitDiagram();
        testObject.doTest();
    }
    
    public abstract String getShortTestName();
    public abstract int getImageWidth();
    public abstract int getImageHeight();
    
    protected int numFilesDrawn = 0;
    protected boolean drawConstraints = false;
    protected boolean drawBoundingBox = false;
    
    /**
     * The image that is written to file on disk.
     */
    protected BufferedImage img = new BufferedImage(getImageWidth(), getImageHeight(), BufferedImage.TYPE_INT_RGB);
    
    /**
     * Do the meat-and-bones of performing the test.
     */
    protected abstract void runTest() throws Exception;
    
    /**
     * Call the functions that perform the test.
     */
    public void doTest()
    {
        try
        {
            System.out.println("Starting test");
            
            deleteLatestFiles();
            runTest();
		    outputDiff();
		    
		    System.out.println("Test complete");
        }
        catch (Exception x)
        {
            System.out.println("Exception whilst testing: " + x.getMessage());
        }
    }
    
    /**
     * Draw the SFile object file to the BufferedImage object img.
     */
    protected void draw(SFile file) throws IOException
    {
        // draw the file's bounding box if set to do so
        if (drawBoundingBox)
            file.drawBoundingBox();
        
        // clear the image
        img.getGraphics().setColor(Color.white);
    	img.getGraphics().fillRect(0, 0, getImageWidth(), getImageHeight());
    	
    	// get the renderer to draw the file onto img's Graphics object
    	SDrawSketch.drawSketch(img.getGraphics(), file, drawConstraints, false);
    	
    	// write the image created by the renderer to a JPEG file on disk
	    ImageIO.write(img, "png", new File(getFileName(numFilesDrawn)));
	    
	    numFilesDrawn++;
    }
    
    /**
     * Output diff.
     */
    protected void outputDiff()
    {
        // TODO
    }
    
    /**
     * Delete the files from the previous latest run.
     */
    protected void deleteLatestFiles()
    {
        File f;
        int i = 0;
        while ((f = new File(getFileName(i++))).exists())
            f.delete();
    }
    
    /**
     * Generates a string of the full path to the image file we want to create.
     * 
     * @param fileNum Which number of file we're drawing.
     * @return The full path of the file to create, as a String.
     */
    protected String getFileName(int fileNum)
    {
        return "../test/latest/" + getShortTestName()
             + (fileNum < 10 ? "00" : (fileNum < 100 ? "0" : ""))
             + Integer.toString(fileNum) + ".png";
    }
    
    /**
     * Write rows of text corresponding to objects, by calling whatIsUnderCursor.
     */
    protected void drawTextMap(SFile f)
    {
        PrintStream console = System.out;
        
        try
        {
            // delete console.txt
            File consoleFile = new File("console.txt");
            consoleFile.delete();
            
            // redirect System.out to console.txt
	        PrintStream psout = new PrintStream(new BufferedOutputStream(new FileOutputStream("console.txt", true)));
	        System.setOut(psout);
	     
	        for (int r = 0; r < getImageHeight() * SDrawSketch.getZoom(); r++)
	        {
	            for (int c = 0; c < getImageWidth() * SDrawSketch.getZoom(); c++)
	            {
	                f.setCursorPosition(c, r);
	                SShape s = f.whatIsUnderCursor();
	                
	                if (s == null)
	                    System.out.print("-");
	                else if (s instanceof SPoint)
	                    System.out.print("P");
	                else if (s instanceof SLine)
	                    System.out.print("L");
	                else if (s instanceof SArc)
	                    System.out.print("A");
	                else if (s instanceof SLabel)
	                    System.out.print("B");
	                else if (s instanceof SInstance)
	                    System.out.print("I");
	                else if (s instanceof SConstraint)
	                    System.out.print("C");
	            }
	            
	            System.out.println();
	            System.out.flush();
	        }
	        
	        // re-redirect System.out to console
	        System.setOut(console);
        }
        catch (FileNotFoundException x)
        {
            System.setOut(console);
            System.out.println("Caught FileNotFoundException in drawTextMap");
        }
    }
}
