import java.io.Serializable;

/**
 * A convenience class to represent absolute co-ordinates that are not subject to
 * constraints (for example, mouse co-ordinatinates). For co-ordinates that ARE subject
 * to constraints, you should use SCoords, which contains two SVariable objects.
 * 
 * @author Tom Craig
 * 
 */
public class SIntPair implements Serializable, Cloneable
{
    public int x;
    public int y;
    
    public SIntPair(int x, int y)
    {
        this.x = x;
        this.y = y;
    }
    public SIntPair(SCoords cToChange)
    {
        this.x = cToChange.getX().getValue();
        this.y = cToChange.getY().getValue();
    }
    
    public void setValues(int x, int y)
    {
    	this.x = x;
    	this.y = y;
	}
 
 
    public int hashCode()
    {
        return x + y;
    }
    
    public boolean equals(Object obj)
    {
        SIntPair rhs = (SIntPair) obj;
        return (rhs.x == x && rhs.y == y);
    }
    
    public String toString()
    {
        return "(" + x + ", " + y + ")";
    }
    
    /**
     * Treat the parameters as 2D co-ordinates and return the square of the distance between them.
     * @param c1 A pair of co-ordinates.
     * @param c2 Another pair of co-ordinates.
     * @return The square of the distance between c1 and c2.
     */
    public static double cartesianDistanceSquared(SIntPair c1, SIntPair c2)
    {
        int dx = c1.x - c2.x;
	    int dy = c1.y - c2.y;
	    double squaredDist = dx * dx + dy * dy;
	    return squaredDist;
    }
    
    /**
     * Treat the parameters as 2D co-ordinates and return the distance between them.
     * @param c1 A pair of co-ordinates.
     * @param c2 Another pair of co-ordinates.
     * @return The distance between c1 and c2.
     */
    public static double cartesianDistance(SIntPair c1, SIntPair c2)
    {
        return Math.sqrt(cartesianDistanceSquared(c1, c2));
    }
    
    /**
    * @returns object reference for cloned object
    */
    public Object clone()
    {
    	Object cln = null;
    	try {
    		cln = super.clone();
    	}
    	catch (CloneNotSupportedException E) {
    		// SConstraint has a clone method so should never reach here
    		System.out.println("Unexpected Error Cloning SShape");
    	}
    	
    	return cln;
    }
}