import java.awt.Graphics;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.awt.Font;
import java.awt.FontMetrics;

/**
 * @author Tom
 */
public class SLabel extends SShape implements Cloneable, Serializable
{
    /**
     * Constructor.
     */
    public SLabel(SPoint topLeft, String text, SFile owner)
    {
        super(topLeft.getLocation() /* shape's handle location is its top left */, owner);
        
        this.topLeft = topLeft;
        this.text = text;
    }
    
    /**
	 * Stores the page co-ordinates of the top left of the instance.
	 */
	public SPoint topLeft = null;
	
	/**
	 * What the label says.
	 */
	private String text;
	
    /**
     * @return Returns the text.
     */
    public String getText()
    {
        return text;
    }
    /**
     * @param text The text to set.
     */
    public void setText(String text)
    {
        this.text = text;
    }
	/**
	 * Get the top left.
	 */
	public SPoint getTopLeft()
	{
	    return topLeft;
	}

    /**
     * @see SShape#getPoints()
     */
    public Collection getPoints()
    {
        ArrayList pts = new ArrayList();
        pts.add(topLeft);
        return pts;
    }
    
    /**
     * @see SShape#getBoundingBox(SIntPair, SIntPair)
     */
    public void getBoundingBox(SIntPair topLeft, SIntPair bottomRight)
    {
		 Font f = SDrawSketch.FONT_USED_FOR_TEXT;
		 g.setFont(f);
		 String s = text;
		 FontMetrics fm   = g.getFontMetrics(f);
		 java.awt.geom.Rectangle2D rect = fm.getStringBounds(s, g);
		 int textHeight = (int)(rect.getHeight())*SDrawSketch.getZoom();
		 int textWidth  = (int)(rect.getWidth())*SDrawSketch.getZoom();
		
		 topLeft.x = this.topLeft.getX().getValue();
		 topLeft.y = this.topLeft.getY().getValue();
		 bottomRight.x = this.topLeft.getX().getValue() + textWidth;
		 bottomRight.y = this.topLeft.getY().getValue() + textHeight;
    }
	
	/**
	 * Returns the distance of the given position from this label.
	*/
	public double minDistanceFromCoords(SIntPair position) {
		SIntPair topLeft = new SIntPair(0,0);
		SIntPair bottomRight = new SIntPair(0,0);
		double distance = 0;
		getBoundingBox(topLeft, bottomRight);
		if((position.x > topLeft.x)&&(position.y > topLeft.y)&&(position.x < bottomRight.x)&&(position.y < bottomRight.y))
		{
			return distance;
		}
		else
		{
			double minX = Math.min(Math.abs(topLeft.x - position.x), Math.abs(bottomRight.x - position.x));
			double minY = Math.min(Math.abs(topLeft.y - position.y), Math.abs(bottomRight.y - position.y));
			distance = Math.min(minX,minY);
		}
		return distance;
	}

    /**
     * @see SShape#getCoordinateRange()
     */
    public Collection getCoordinateRange()
    {
        // TODO This coordinate range is pathetic! At the moment the user has to move very
        // near the top left to select. Do a better one!
        
        ArrayList coordRange = new ArrayList();
        coordRange.add(new SIntPair(topLeft.getX().value(), topLeft.getY().value()));
        return coordRange;
    }
    
    /**
    * @returns object reference for cloned object
    */
    public Object clone()
    {
    	SLabel cln = null;
    	
    	cln = (SLabel) super.clone();
    	
    	cln.topLeft = (SPoint) this.topLeft.clone();
    	cln.text = this.text.substring(0, this.text.length());
    	
    	return cln;
    }
    

    /**
     * Nasty hack!
     */
    public static Graphics g;

}
