import javax.swing.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.*;
import java.awt.datatransfer.*;
import java.awt.image.*;


public class SLibraryPanel extends JPanel
{
	DragSource dragSource = null;
	JScrollPane sp;
	FileThumbnail ft;
	InnerPanel ip;
	SLibrary library;
	SketchUI parent;
				
	
	public SLibraryPanel(SLibrary library,SketchUI parent)
	{
		this.parent = parent;
		this.library = library;
		ft = new FileThumbnail();
		ft.setBorder(BorderFactory.createMatteBorder(1,1,1,1, new Color(0x666666)));
		this.add(ft);
		
		System.out.println(ft.getWidth());
		System.out.println(ft.getHeight());
		
		ip = new InnerPanel();
		sp = new JScrollPane(ip);
		this.setPreferredSize(new Dimension(180,381));
		
		sp.setPreferredSize(new Dimension(160,180));
		this.add(sp);

		JButton editButton = new JButton("Edit Instance");
		editButton.addActionListener (new ActionListener ()
		{
			public void actionPerformed (ActionEvent ae)
			{
				editInstance();
			}
		});
				

		this.add(editButton);
		
		
		
	}
	
	private void editInstance(){
		if(ip.getActive()!=null){
			parent.openFile(ip.getActive().getText());
		}
		
	}
		
	/**
	 * updates the inner panel based on which files are now in the library
     */
	public void updateList()
	{
		this.ip.libraryFiles = SLibrary.getLibFilesList();

		this.ip.updateEntries();
	}
			
	class InnerPanel extends JPanel 
	{
		public String[] libraryFiles = SLibrary.getLibFilesList();
			
		ArrayList entries;
		ImgLabel active;
		ImgLabel dragged;
		
		// used for double buffering the thumbnail
		BufferedImage offscreen;
		Graphics buffer;

		public InnerPanel()
		{
									
			setLayout(new GridLayout(0,1)); // 2 rows 1 column
			active = null;
			entries = new ArrayList();
			//setSize(140,0);
			
					
			updateEntries();				
			
			// set the first file active by default
			if(libraryFiles.length != 0)
				setActive((ImgLabel) entries.get(0));	
			
		}
		
		/**
		 * update the inner pannel with which files are in the library
		 */
		public void updateEntries() 
		{
			this.dragged = null;
			libraryFiles = SLibrary.getLibFilesList();
			entries.clear();
			
			this.removeAll();
			int i;
			for(i=0;i<libraryFiles.length;i++)
			{
				entries.add(new ImgLabel(libraryFiles[i]));
				add((ImgLabel) entries.get(i));
			}
			this.setPreferredSize(new Dimension(130,i*40));
			this.revalidate();
			
			ft.setBackground((libraryFiles.length == 0)?Color.LIGHT_GRAY:Color.WHITE);	

		}
				
				
		void setActive(ImgLabel label)
		{
			if (active!=null)
			{
				active.changeState(false);			
			}
			label.changeState(true);
			active = label;
			
			updateThumb();	 // udpate the thumbnail with the currenlty selected file
						
		}
		
		public ImgLabel getActive()
		{
			return active;
		}
		
		/**
		 * update the thumbnail with the currently selected file
		 */		
		public void updateThumb()
		{

			try{
			// draw the image onto an offscreen buffer				
			offscreen = (BufferedImage) createImage(ft.getWidth()-20, ft.getHeight()-20);
			buffer = offscreen.getGraphics();				
			SDrawSketch.drawSketch(buffer, library.getFile(active.name.getText()), false, true);
			// draw the image to the thumbnail canvas
			ft.getGraphics().drawImage(offscreen, 10, 10, this);
			}
			catch(Exception e)
			{}
		}
		
	}
	
	
	/**
	 * the thumbnail panel
	 */
	class FileThumbnail extends JPanel {
		
		// set up the thumbnail panel			
		public FileThumbnail()
		{
			setBackground(Color.WHITE);
			setPreferredSize(new Dimension(160,160));
			setSize(160, 160);
			DragSource dragSource = DragSource.getDefaultDragSource();
			
			// component, action, listener
			dragSource.createDefaultDragGestureRecognizer(this,DnDConstants.ACTION_COPY,new DragObject()); 
		}
	}
	
	class ImgLabel extends JPanel  implements MouseListener
	{
		boolean mouseOver;
		JLabel name;
		String url;
		
		public ImgLabel (String url)
		{
			setBackground(new Color(0xFFFFFF));
			setLayout (new FlowLayout(FlowLayout.LEFT));
			setPreferredSize(new Dimension(130,40));
			setSize(130,40);
			
			this.url = url;
			name = new JLabel (url);
			name.setFont (new Font("Sans",Font.PLAIN,11));
			this.add(name);
			this.addMouseListener(this);
			
			DragSource dragSource = DragSource.getDefaultDragSource();
			
			// component, action, listener
			dragSource.createDefaultDragGestureRecognizer(this,DnDConstants.ACTION_COPY,new DragObject()); 
		}
		
		public void mouseClicked(MouseEvent me){}
		public void mousePressed(MouseEvent me){setMeActive();}
		public void mouseReleased(MouseEvent me){}
		public void mouseEntered(MouseEvent me){mouseOver = true;}
		public void mouseExited(MouseEvent me){mouseOver = false;}
		
		void setMeActive ()
		{
			try{
				//SFile f = library.openFile(url);
				//Graphics iconG = this.getGraphics();
				//SDrawSketch.drawSketch(iconG,f,false,true);
				int x = 5;
				if (x < 4) throw new SFileAlreadyOpenException("");
				else if (x >6) throw new SFileNotInLibraryException();
				
			}
			catch (SFileAlreadyOpenException e)
			{
			
			}
			catch (SFileNotInLibraryException e)
			{
				
			}
			ip.setActive(this);
		}

		public String getText(){
			return url;
		}
		
		void changeState(boolean selected){
			if(selected)
			{
				setBackground(new Color(0x000099));
				name.setForeground(new Color(0xFFFFFF));
			}
			else
			{
				setBackground(new Color(0xFFFFFF));
				name.setForeground(new Color(0x000000));
			}
		}
		
		
	}
	
	class DragObject implements DragGestureListener,DragSourceListener{
		public void dragGestureRecognized(DragGestureEvent e) {
			System.out.println(e.getDragAction());
			if((e.getDragAction() & DnDConstants.ACTION_COPY) == 0) return;
			Transferable transferable = new StringSelection(ip.getActive().getText());
			try
			{
				e.startDrag(DragSource.DefaultCopyNoDrop,transferable,this);
			}
			catch(InvalidDnDOperationException idoe)
			{
				System.err.println( idoe );
			}
		}

			
		public void dragDropEnd(DragSourceDropEvent e) {
			if( e.getDropSuccess() == false )
			{
				System.out.println( "not successful");
				return;
			}
		}

		public void dragEnter(DragSourceDragEvent e) {
			System.out.println( "draglabel enter " + e);
			DragSourceContext context = e.getDragSourceContext();
			int myaction = e.getDropAction();
			if((myaction & DnDConstants.ACTION_COPY) != 0)
			{	
				context.setCursor(DragSource.DefaultCopyDrop);		
			}
			else
			{
				context.setCursor(DragSource.DefaultCopyNoDrop);			
			}
		}
		
		
		public void dragOver(DragSourceDragEvent e) {}

		public void dragExit(DragSourceEvent e) {}

		public void dropActionChanged (DragSourceDragEvent e) {
			DragSourceContext context = e.getDragSourceContext();		
			context.setCursor(DragSource.DefaultCopyNoDrop);			
		}
	}
}

