import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.*;

/**
 * Woah! There's so much going on here I don't know where to start.
 *
 * @author Tom
 */
public class SPoint extends SShape implements Cloneable, Serializable
{

	/**
	 * Contains the offset for the copy and paste operation 
	 */	
	private static SIntPair cpoffset = new SIntPair(0,0);

	/**
	 * The location of the point in page co-ordinates. Equal to its handle location.
	 */
	private SCoords location;
	
	/**
	 * Whether this point is fixed - which means it cannot be moved by the constraints solver.
	 */
	private boolean fixed;
	
	/**
	 * True if point an attacher of some instance.
	 */
	private boolean isAttacher = false;
	
	/**
	 * A hashmap of points that have already been copied during a copy and paste operation
	 */
	private static HashMap CopyPoints = new HashMap();
	
	/**
	 * A default constructor.
	 */
	public SPoint(int x, int y, SFile owner)
	{
	    super(new SCoords(x, y), owner);
	    this.location = this.handleLocation;	// which we've just set in the call to 'super'
	    
	    //System.out.println("In SPoint(int, int) with x = " + Integer.toString(x) + ", y = " + Integer.toString(y));
	}
	
	/**
	 * A constructor for when we have a SCoords object to provide already.
	 */
	public SPoint(SCoords location, SFile owner)
	{
	    super(location, owner);
	    this.location = location;
	}
	
	/**
	 * Another constructor, for convenience.
	 */
	public SPoint(SIntPair c, SFile owner)
	{
	    super(new SCoords(c.x, c.y), owner);
	    this.location = this.handleLocation;	// which we've just set in the call to 'super'
	    
	    /*System.out.print("In SPoint(SIntPair c), with c = ");
	    c.print();
	    System.out.println("");*/
	}
	
	/**
	 * A constructor taking whether point is an attacher.
	 */
	public SPoint(SCoords location, SFile owner, boolean isAttacher)
	{
	    super(location, owner);
	    this.location = location;
	    this.isAttacher = isAttacher;
	}
	
	public SCoords getLocation() {
	    return location;
	}
	
	public SVariable getX() {
		return location.getX();
	}
	
	public SVariable getY() {
		return location.getY();
	}
	
	
		
	/**
	 * Given a collection of shapes that might contain this point, this method
	 * returns those that do contain it.
	 * 
	 * This method is used in at least:
	 * 		<li>Matthew's code for merging;
	 * 		<li>Tom C's code for constraining the endpoints of an arc to lie on that arc.
	 * 
	 * @param possibleDependentShapes The shapes of which we return a subset.
	 * @return All shapes in possibleDependentShapes containing this point.
	 */
	public Collection getDependentShapes(Collection possibleDependentShapes)
	{
	    Collection definites = new ArrayList();	// we'll return this
	    
	    Iterator i = possibleDependentShapes.iterator();
	    while (i.hasNext())
	    {
	        SShape s = (SShape) i.next();
	        if (!(s instanceof SConstraint)){
	        	//if one of the shape s's points is 'this', then add s to the collection to be returned
		        if (s != this && (s.getPoints() != null) && s.getPoints().contains(this))
		            definites.add(s);
		            
		                        		            
	        }
	        else
	        {
	        	SConstraint sc = (SConstraint) s;
	        	if (sc.getLinkedPoints() != null && sc.getLinkedPoints().contains(this))
		            definites.add(s);
	        }
	        
	    }
	    
	    return definites;
	}

	/**
	 * @see SShape#getBoundingBox(SIntPair, SIntPair)
	 */
	public void getBoundingBox(SIntPair topLeft, SIntPair bottomRight)
	{
	    // OK, this is easy: the bounding box of a point is the coords of the point itself
	    topLeft.x = bottomRight.x = location.getX().value();
	    topLeft.y = bottomRight.y = location.getY().value();
	}
	
	/**
	 * @return The co-ordinates occupied by this point, i.e. the co-ordinates of the point!
	 */
	public Collection getCoordinateRange()
	{
	    ArrayList a = new ArrayList();
	    a.add(location.makeIntPair());
	    return a;
	}

	/**
	 * @return A list of co-ordinates occupied by this shape.
	 */
	public Collection getPoints()
	{
	    ArrayList pts = new ArrayList();
	    pts.add(this);
	    return pts;
	}
	
    /**
     * @see SShape#moveShape(int, int)
     */
    public void moveShape(int dx, int dy)
    {
        super.moveShape(dx, dy);
        
        // am I an instance handle? if so, move instance attachers with me
        SInstance instance = null;
        Iterator i = owner.myShapes().iterator();
        while (i.hasNext())
        {
            SShape s = (SShape) i.next();
            if (s instanceof SInstance && ((SInstance) s).getLocation() == this)
                instance = (SInstance) s;
        }
        if (instance != null)
        {
            ArrayList instanceAttachers = (ArrayList) instance.getAttachers();
            for (int j = 0; j < instanceAttachers.size(); j++)
                ((SPoint) instanceAttachers.get(j)).moveShape(dx, dy);
        }
    }
    
    /**
     * @returns collection of all points that have been copied
     */
	public static Collection getCopyPoints()
	{
		return CopyPoints.values();
	}
	
    /**
     * @return Returns whether point is fixed.
     */
    public boolean isFixed()
    {
        return fixed;
    }
	
    /**
     * @return Returns whether point is the attacher of some instance.
     */
    public boolean isAttacher()
    {
        return isAttacher;
    }
    
    /**
     * @param fixed Sets whether point is fixed.
     */
    public void setFixed(boolean fixed)
    {
        this.fixed = fixed;
        location.getX().fixed = fixed;
        location.getY().fixed = fixed;
    }
    
    /**
     * Empties the CopyPoints hashmap for a new copy and paste operation
     */
    public static void emptyCpyPts()
    {
    	CopyPoints.clear();
    }
    
    /**
    * @returns object reference for cloned object
    */
    public Object clone()
    {
    	// work out new co-ordinates of point based on offset
    	SIntPair newcoords = new SIntPair(this.location.getX().value() + cpoffset.x, 
	 														  this.location.getY().value() + cpoffset.y);
	 		 		 		 	
 		String cpEntry = this.location.toString(); // get the location of the string
		SPoint cln = null;

		// if the collection of points already has copied this point then just use
		// that point instead of cloning			
		if (CopyPoints.containsKey(cpEntry))
		{
			 // if the point is already in the HashMap, just use that
			 cln = (SPoint) CopyPoints.get(cpEntry);    
		}
		
		// otherwise just clone the point and add it the the collection of
		// copied points
		else
		{
			cln = (SPoint) super.clone();
			cln.location = (SCoords) this.location.clone();
					
			// change the location of the coordinate to its required position
			cln.moveShapeAbs(newcoords.x, newcoords.y);
					
			// add the point to the HashMap
			CopyPoints.put(this.location.toString(), cln);
		}
           	
       	return cln;
    }
    
    /**
    * Sets the offset for all the points in the copy and paste operation
    */
    public static void setOffset(SIntPair offset)
    {
    	cpoffset.x = offset.x;
    	cpoffset.y = offset.y;
    }
    
    public static int getOffsetX() { return cpoffset.x; }
    public static int getOffsetY() { return cpoffset.y; }
	
	/**
	 * Returns the distance of the given position from this point.
	*/
	public double minDistanceFromCoords(SIntPair position) {
		int dx = position.x - location.getX().value();
		int dy = position.y - location.getY().value();
		return java.lang.Math.sqrt((double) (dx*dx + dy*dy));
	}
    
}