import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import javax.imageio.ImageIO;

/**
 * Draws a circuit diagram. Similar to the Operational Scenario defined in the spec.
 * 
 * @author Tom Craig
 */
public class STCircuitDiagram extends SDataTest
{
    public String getShortTestName()	{ return "circuit"; }
    public int getImageWidth()			{ return 800; }
    public int getImageHeight()			{ return 600; }
    
    /**
     * @see SDataTest#runTest()
     */
    protected void runTest() throws Exception
    {
        /* this code was simply to test that the Graphics object works - which it does!
		Graphics2D gfx = (Graphics2D) bimg.getGraphics();
        gfx.drawLine(3, 4, 125, 457);
        gfxs.draw(new Line2D.Double(3, 4, 100, 200));*/
        
        SDrawSketch.setPageCoordinates(new SCoords(0,0));
        
        // -- BULB --
        
        SFile bulb = new SFile();
        
        // create left line
        bulb.setCursorPosition(50, 111);
        SPoint bp1 = bulb.createPoint();
        bulb.commitShape(bp1);
        bulb.setCursorPosition(200, 111);
        SPoint bp2 = bulb.createPoint();
        bulb.commitShape(bp2);
        SLine bl1 = bulb.createLine(bp1, bp2);
        bulb.commitShape(bl1);
        
        draw(bulb);
        
        // create right line
        bulb.setCursorPosition(300, 111);
        SPoint bp3 = bulb.createPoint();
        bulb.commitShape(bp3);
        bulb.setCursorPosition(450, 111);
        SPoint bp4 = bulb.createPoint();
        bulb.commitShape(bp4);
        SLine bl2 = bulb.createLine(bp3, bp4);
        bulb.commitShape(bl2);
        
        draw(bulb);
        
        // create semicircle
        bulb.setCursorPosition(250, 111);
        SPoint centre = bulb.createPoint();
        bulb.commitShape(centre);
        SArc ba1 = bulb.createArc(centre, bp2, bp3, true /* anticlockwise */);
        bulb.commitShape(ba1);
        
        draw(bulb);
        
        // create circle
        bulb.setCursorPosition(175, 111);
        SPoint bp5 = bulb.createPoint();
        bulb.commitShape(bp5);
        bulb.setCursorPosition(325, 111);
        SPoint bp6 = bulb.createPoint();
        bulb.commitShape(bp6);
        SArc ba2 = bulb.createArc(centre, bp5, bp5, true);
        bulb.commitShape(ba2);
        
        draw(bulb);
        
        // do a custom draw
        SDrawSketch.drawCoordRange(img.getGraphics(), ba2);
        ImageIO.write(img, "jpg", new File(getFileName(numFilesDrawn)));
        numFilesDrawn++;
        // end of custom draw
        
        // set the attachers
        bulb.setAttacherState(bp1, true);
        bulb.setAttacherState(bp4, true);
        
        draw(bulb);
        //drawTextMap(bulb);
        
        // -- CIRCUIT --
        
        SFile circuit = new SFile();
        
        // create instance
        circuit.setCursorPosition(275, 400);	// N.B. renderer misbehaves at certain first args (e.g. 300)
        SInstance ci1 = circuit.createInstance(bulb);
        circuit.commitShape(ci1);
        ci1.setScaleFactor(0.5);
        
        draw(circuit);
        
        // get its attachers
        Collection attachers = ci1.getAttachers();
        Iterator i = attachers.iterator();
        SPoint cattl = (SPoint) i.next();
        SPoint cattr = (SPoint) i.next();
        System.out.print("cattl: ");
        cattl.getLocation().print();
        System.out.println();
        
        draw(circuit);
        
        // create left wire to bulb
        circuit.setCursorPosition(150, 455);
        SPoint cp1 = circuit.createPoint();
        circuit.commitShape(cp1);
        SLine cl1 = circuit.createLine(cp1, cattl);
        circuit.commitShape(cl1);	      
        
        draw(circuit);  
        
        // create right wire to bulb
        circuit.setCursorPosition(625, 455);
        SPoint cp2 = circuit.createPoint();
        circuit.commitShape(cp2);
        SLine cl2 = circuit.createLine(cp2, cattr);
        circuit.commitShape(cl2);
        
        draw(circuit);
        
        // fix attachers and constrain wire distances
        /*	-- This constraint doesn't seem to be working quite properly at the moment
			   (uncomment it out to see)
		circuit.setFixedState(cattl, true);
        circuit.setFixedState(cattr, true);
        ArrayList cc3args = new ArrayList();
        cc3args.add(cattl);
        cc3args.add(cp1);
        cc3args.add(new SVariable(60));
        SCSetDistance cc3 = (SCSetDistance) circuit.createConstraint(cc3args, SCSetDistance.class);
        circuit.commitShape(cc3);*/
        
        // create left vertical wire
        circuit.setCursorPosition(159, 255);
        SPoint cp3 = circuit.createPoint();
        circuit.commitShape(cp3);
        SLine cl3 = circuit.createLine(cp1, cp3);
        circuit.commitShape(cl3);
        
        draw(circuit);
        
        // constrain to be vertical
        circuit.setFixedState(cp1, true);
        ArrayList cc1args = new ArrayList();
        cc1args.add(cp1);
        cc1args.add(cp3);
        SCSameX cc1 = (SCSameX) circuit.createConstraint(cc1args, SCSameX.class);
        circuit.commitShape(cc1);
        
        draw(circuit);
        
        // create right vertical wire
        circuit.setCursorPosition(600, 255);
        SPoint cp4 = circuit.createPoint();
        circuit.commitShape(cp4);
        SLine cl4 = circuit.createLine(cp2, cp4);
        circuit.commitShape(cl4);
        
        draw(circuit);
        
        // constrain to be vertical
        circuit.setFixedState(cp2, true);
        ArrayList cc2args = new ArrayList();
        cc2args.add(cp2);
        cc2args.add(cp4);
        SCSameX cc2 = (SCSameX) circuit.createConstraint(cc2args, SCSameX.class);
        circuit.commitShape(cc2);
        
        draw(circuit);
        
        // create last (top) line
        SLine cl5 = circuit.createLine(cp3, cp4);
        circuit.commitShape(cl5);
        
        draw(circuit);
        
        // create a label
        circuit.setCursorPosition(100, 100);
        SPoint cp5 = circuit.createPoint();
        SLabel clbl1 = circuit.createLabel(cp5, "Sketchpad in action!");
        circuit.commitShape(clbl1);
        
        draw(circuit);
        
        // satisfy the constraints
        Collection constraints = new ArrayList();
        constraints.add(cc1);
        constraints.add(cc2);
        //SConstraintSolver.satisfyConstraints(constraints, 200);
        
        draw(circuit);
        //drawTextMap(circuit);
    }
}
