import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * @author Matthew Brown
 *
 * This class generates a toolbar for use in
 * the Sketchpad utility.
 * 
 */
public class SToolBar extends JPanel
{
	//the array of btn objects
	SButton tools[];
	SButton[] subButtons;

	//set up a list of icon btn urls
	static String buttonInfo[][] = 
		{
			{"images/cursor.png","Cursor"},
			{"images/move.png","Move Item"},
			{"images/drawline.png","Draw Line"},
			{"images/drawarc.png","Draw Arc"},
			{"images/type.png","Add Text"},
			{"images/number.png","Add Number Constraint"},
			{"images/constraint.png","Add Constraint"},
			{"images/attacher.png","Set Attacher"},
			{"",""},
			{"",""},
			{"images/rotate.png","Rotate Instance"},
			{"images/grow.png","Resize Instance"},
			{"images/lock.png","Lock Point"},
			{"",""},
			{"",""},
			{"",""},
			{"images/copy.png","Copy"},
			{"images/paste.png","Paste"},
			{"",""},
			{"",""},
			{"images/move-16.png","Navigate workspace"},
			{"images/magnify.png","Zoom"}
		};

	static Class [] constraintClasses =
		{
			SCCollinear.class,
			SCDistanceFactor.class,
			SCErect.class,
			SCFixedDistance.class,
			SCInstanceIsOriginalSize.class,
			SCInstanceIsXTimesOriginal.class,
			SCInstanceIsXTimesSmallerThanOriginal.class,
			SCOnItsSide.class,
			SCParallel.class,
			SCPerpendicular.class,
			SCSameLength.class,
			SCSameX.class,
			SCSameY.class,
			SCScaleFactor.class,
			SCSetDistance.class,
			SCSetSize.class,
			SCVarIsDistanceDelta.class,
			SCVarIsDistance.class,
			SCSameDistance.class
		};
	static String [] [] constraintURLs = {
		{imgfilename(SCCollinear.getSymbol()), SCCollinear.getDescription()},
		{imgfilename(SCDistanceFactor.getSymbol()), SCDistanceFactor.getDescription()},
		{imgfilename(SCErect.getSymbol()), SCErect.getDescription()},
		{imgfilename(SCFixedDistance.getSymbol()), SCFixedDistance.getDescription()},
		{imgfilename(SCInstanceIsOriginalSize.getSymbol()), SCInstanceIsOriginalSize.getDescription()},
		{imgfilename(SCInstanceIsXTimesOriginal.getSymbol()), SCInstanceIsXTimesOriginal.getDescription()},
		{imgfilename(SCInstanceIsXTimesSmallerThanOriginal.getSymbol()), SCInstanceIsXTimesSmallerThanOriginal.getDescription()},
		{imgfilename(SCOnItsSide.getSymbol()), SCOnItsSide.getDescription()},
		{imgfilename(SCParallel.getSymbol()), SCParallel.getDescription()},
		{imgfilename(SCPerpendicular.getSymbol()), SCPerpendicular.getDescription()},
		{imgfilename(SCSameLength.getSymbol()), SCSameLength.getDescription()},
		{imgfilename(SCSameX.getSymbol()), SCSameX.getDescription()},
		{imgfilename(SCSameY.getSymbol()), SCSameY.getDescription()},
		{imgfilename(SCScaleFactor.getSymbol()), SCScaleFactor.getDescription()},
		{imgfilename(SCSetDistance.getSymbol()), SCSetDistance.getDescription()},
		{imgfilename(SCSetSize.getSymbol()), SCSetSize.getDescription()},
		{imgfilename(SCVarIsDistanceDelta.getSymbol()), SCVarIsDistanceDelta.getDescription()},
		{imgfilename(SCVarIsDistance.getSymbol()), SCVarIsDistance.getDescription()},
		{imgfilename(SCSameDistance.getSymbol()), SCSameDistance.getDescription()}
		
		};
	
	
	public static String imgfilename(String symbol) { return "images/constraint" + symbol.toLowerCase() + ".png"; }

	LittleBar barlet;
	ConstraintsFrame constraintsFrame;
	SToolTips tooltips;

	// the integer index of the current tool
	int currentTool;

	SketchUI parent;

	GridBagConstraints constraints = new GridBagConstraints();

	public SToolBar (SToolTips tooltips, SketchUI parent)
	{

		// Sets preferred size
		setPreferredSize (new Dimension(60,400));

		this.parent = parent;
		JPanel toolGrid = new JPanel();

		// Sets the layout manager
		toolGrid.setLayout (new GridBagLayout());

		this.tooltips = tooltips;

		// Sets up the button array
		tools = new SButton[buttonInfo.length];
		int x,y;

		// Code to add buttons to the window

		int top = 0, left = 0, bottom = 0, right = 0;

		constraints.insets = new Insets(top,left,bottom,right);
		for (int i=0;i<tools.length;i++)
		{
			if (!buttonInfo[i][0].equals(""))
			{
				tools[i] = new SButton(buttonInfo[i][0],buttonInfo[i][1], i, false, this);
				addGB(toolGrid,tools[i],x=(i%2),y=i/2);
				top = -1;
				left = (i%2)-1;
				constraints.insets = new Insets(top,left,bottom,right);
			}
			else
			{
				if (i%2==0) top=10;
				constraints.insets = new Insets(top,left,bottom,right);
			}
		}

		// Sets the default tool to the cursor (index 0)
		currentTool = 0;
		setActive(0,false,true,"Cursor");
		repaint();

		addMouseListener (new MouseAdapter ()
		{
			public void mouseExited(MouseEvent me)
			{
				clearTips();
			}
		});

		barlet = new LittleBar (this);
		constraintsFrame = new ConstraintsFrame(this);
		
		add(toolGrid);
		add(barlet);
	}

	private void clearTips()
	{
		tooltips.setText("");
	}
	
	/*
	 * Resets the current tool to the cursor. Called by SketchUI 
	 */
	public void resetTool()
	{
		setActive(0,false,true,"Cursor");
	}
	
	/*
	 * Returns the currently selected tool
	 */
	public String getSelectedTool()
	{
		return buttonInfo[currentTool][0];
	}

	protected void addGB (Container cont,Component component, int x, int y)
	{
		constraints.fill = GridBagConstraints.NONE;
		constraints.anchor = GridBagConstraints.SOUTHEAST;
		constraints.gridx = x;
		constraints.gridy = y;
		cont.add(component, constraints);
	}

	/*
	 * Sets the active tool
	 * @param tool the index of the new tool
	 * @param subMenu indicates whether the tool is within a sub-menu or not
	 */
	protected void setActive (int newTool, boolean subMenu, boolean clickedOn, String description)
	{
		if (clickedOn)
		{
			if (!subMenu)
			{
				if (newTool==currentTool) return;

				tools[currentTool].changeState(0);
				tools[newTool].changeState(1);
		
				parent.clickedOn(description);
				System.out.println(newTool);
				barlet.clean();
				currentTool = newTool;
				if (newTool==6||newTool==21)
				{
					if (newTool==6){
						constraintsFrame.show();
						constraintsFrame.setActive(0);
					}
					else if (newTool==21){
						barlet.setVisible(true);
						parent.clickedOn("Zoom In");
					}
				}
			}
			else
			{
				//do other code
				if(newTool>=100){
					constraintsFrame.setActive(newTool-100);
					this.setActive(6,false,true,"Add Constraint");
					parent.constraintClickedOn(constraintClasses[newTool-100]);
				}
				else if(newTool>=50){
					subButtons[newTool-50].changeState(1);
					subButtons[1-(newTool-50)].changeState(0);
					parent.clickedOn(description);
				}
			}
		}
		else
		{
				
			tooltips.setText(description);
		}

	}
	
	protected void setActive (int newTool, boolean subMenu, boolean clickedOn, String description, String constraint)
	{
		System.out.println(constraint);
		setActive (newTool, subMenu, clickedOn,description);
	}

	// A class to draw the sub-menus to screen
	class LittleBar extends JPanel
	{
		SToolBar toolbar;
		String magnifyURLs[][]=
		{
			{"images/zoomin.png","Zoom In"},
			{"images/zoomout.png","Zoom Out"}
		};
		
		/*
		 * The LittleBar constructor
		 * @param btnURLs an array containing the image URLs for the sub-menu
		 */
		public LittleBar(SToolBar toolbar)
		{
			this.setLayout (new GridBagLayout());
			this.setSize(55,40);
			this.setPreferredSize(new Dimension(55,40));
			this.toolbar = toolbar;
			
			subButtons = new SButton[2];
			int extra = 50;
			for (int i=0;i<magnifyURLs.length;i++)
			{
				subButtons[i] = new SButton(magnifyURLs[i][0],magnifyURLs[i][1],i+extra, true, toolbar);
				constraints.insets = new Insets(0,-1,-1,0);
				subButtons[0].changeState(1);
				toolbar.addGB(this,subButtons[i],(i)%2,(i)/2);
			}
			revalidate();
			repaint();
			this.setVisible(false);
		}

		public void clean()
		{
			subButtons[0].changeState(1);
			subButtons[1].changeState(0);
			System.out.println(this);
			this.setVisible(false);
		}
	}
		
	class ConstraintsFrame extends JFrame
	{
		SToolBar toolbar;
		SButton[] constraintButtons;
		JPanel cont;
		
		public ConstraintsFrame(SToolBar toolbar)
		{
			super("Constraints");
			cont = new JPanel();
			constraintButtons = new SButton[19];
			cont.setLayout (new GridBagLayout());
			cont.setPreferredSize(new Dimension(180,100));
			this.getContentPane().add(cont);
			this.setSize(180,100);
			this.setLocation(500,200);
			this.show();
			this.toolbar = toolbar;
			
//			draw constraints into the panel
			for(int i=0;i<SToolBar.constraintURLs.length;i++)
			{
				//System.out.println(i);
				constraintButtons[i] = new SButton(constraintURLs[i][0],constraintURLs[i][1],i+100, true, toolbar);
				constraints.insets = new Insets(0,-1,-1,0);
				toolbar.addGB(cont,constraintButtons[i],(i)%7,(i)/7);
			}
			revalidate();
			this.hide();
		}
		
		public void setActive(int a)
		{
			for(int i=0;i<SToolBar.constraintURLs.length;i++)
			{
				constraintButtons[i].changeState(1);
				constraintButtons[i].changeState(0);
			}
			if(a>=0)constraintButtons[a].changeState(1);
		}
	}
}