import java.util.Collection;
import java.io.Serializable;
import java.util.TreeSet;


/**
 * This is a generic variable for the data system. It can be manipulated by constraints, and keeps track of what constraints it has.
 * Apart from that, it's basically a wrapper for the Integer class.
 *
 * @author David Stark
 * @version 1.2
 * @composed - - * SConstraint
*/

public class SVariable implements Cloneable, Serializable
{

/**
 * The actual integer value of this variable.
*/
private int value;

/**
 * The value before constraint satisfaction - used for undo.
*/
private int cachedValue;

/**
 * Whether the variable is fixed, that is, immutable by the constraints solver.
*/
public boolean fixed;

/**
 * The constraints of this variable.
*/
public TreeSet myConstraints;

/**
 * A default constructor.
*/
public SVariable(int value) {
	myConstraints = new TreeSet();
	this.value = value;
}

/**
 * Caches its current value in a different variable, so it can be re-set if constraint satisfaction goes awry.
*/
public void cacheValue() {
	cachedValue = value;
}

/**
 * Resets its value to the one in the cache, undoing any changes made since cacheValue() was called.
*/
public void resetFromCache() {
	value = cachedValue;
}

/**
 * @returns a collection of the constraints acting upon this variable
*/
public Collection myConstraints() {
	return myConstraints;
}

/**
 * Adds a new constraint to the set of constraints binding this variable.
 * Note that if a constraint refers to this variable "for reference only", it should not be registered here.
 *
 * @param theC the new constraint
*/
public void addConstraint(SConstraint theC) {
	myConstraints.add(theC);
}

/**
 * Removes a constraint from the set of constraints binding this variable.
 *
 * @param theC the constraint to remove
*/
public void removeConstraint(SConstraint theC) {
	myConstraints.remove(theC);
}

/**
 * @returns the integer value of this variable
*/
public int value() {
	return value;
}

/**
 * @returns the integer value of this variable
*/
public int getValue() {
	return value;
}

/**
 * Sets the integer value of this variable.
 *
 * @param value the new value
*/
public void setValue(int value) {
	this.value = value;
}

	/**
    * @returns object reference for cloned object
    */
    public Object clone()
    {
    	SVariable cln = null;
    	try {
    		cln = (SVariable) super.clone();
    		System.out.println("Cloning SVARS");
    	}
    	catch (CloneNotSupportedException E) {
    		// SConstraint has a clone method so should never reach here
    		System.out.println("Unexpected Error Cloning SCDistanceFactor");
    	}
    	
    	cln.myConstraints = (TreeSet) this.myConstraints.clone();
    	
    	return cln;
    }

}
