import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * This class stores information about variables being one and the same for purposes of constraints evaluation.
 *
 * @author David Stark
*/
public class SVariableUnion {
	private HashMap slaveToMaster;
	private ArrayList masterVars;
	
	/**
	 * The constructor. It gets fed an arraylist of arraylists of SVariables that are the same variable,
	 * and assigns them master/slave status, sets the master's value, and fills its hashmap.
	*/
	public SVariableUnion(ArrayList varUnions) {
		//set up the storage variables
		masterVars = new ArrayList();
		slaveToMaster = new HashMap(varUnions.size() * varUnions.size() * 2); //an approximation of the number of slaves
	
		//now iterate through the individual union statements
		Iterator unionsIter = varUnions.iterator();
		while (unionsIter.hasNext()) {
			ArrayList currentUnion = (ArrayList) unionsIter.next();
			Iterator currentUnionIter = currentUnion.iterator();
			
			boolean fixedMaster = true;
			
			//check if the union has not got a fixed master
			if (currentUnion.get(0) == null) {
				//if it doesn't, we need to get rid of the initial null
				fixedMaster = false;
				currentUnionIter.next();
				System.out.println("master not fixed");
			}
			
			//declare the first variable of the union the master
			SVariable master = (SVariable) currentUnionIter.next();
			masterVars.add(master);
			
			double averageVal = (double) master.value();
			
			//now loop through all slave variables, adding their values to the average, and filling the hashmap
			while (currentUnionIter.hasNext()) {
				SVariable slave = (SVariable) currentUnionIter.next();
				averageVal += (double) slave.value();
				slaveToMaster.put(slave, master);
			}
			
			//if it is not fixed, set the master to the average of all values
			if (fixedMaster == false) {
				averageVal /= (currentUnion.size() - 1);
				master.setValue((int) averageVal);
			}
		}
	}
	
	/**
	 * This function, given a variable, will return a suitable substitute to use in re-making constraints.
	*/
	public SVariable returnSubstitute(SVariable theVar) {
		if (slaveToMaster.containsKey(theVar)) {
			return (SVariable) slaveToMaster.get(theVar);
		}
		return theVar;
	}
	
	/**
	 * This method will sets values of all slave variables to the master variable's value.
	 * It is called at the end of constraint evaluation.
	*/
	public void setSlaves() {
		System.out.println("Setting slaves...");
		Iterator iter = slaveToMaster.keySet().iterator();
		while (iter.hasNext()) {
			SVariable slave = (SVariable) iter.next();
			System.out.print("Variable Union Setting: ");
			System.out.print(slave.value());
			System.out.print(" to ");
			System.out.println(((SVariable) slaveToMaster.get(slave)).value());
			slave.setValue(((SVariable) slaveToMaster.get(slave)).value());
		}
	}
}
